@@ -8,13 +8,10 @@
 #ifdef	FD
 #include "fd.h"
 #else	/* !FD */
+#include "machine.h"
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
-#include "machine.h"
-#include "printf.h"
-#include "kctype.h"
-#include "pathname.h"
 
 #ifndef	NOUNISTDH
 #include <unistd.h>
@@ -23,15 +20,16 @@
 #ifndef	NOSTDLIBH
 #include <stdlib.h>
 #endif
+
+#include "printf.h"
+#include "kctype.h"
 #endif	/* !FD */
 
 #if	MSDOS
-#include <io.h>
 #include "unixemu.h"
-#else	/* !MSDOS */
+#else
 #include <sys/param.h>
-#include <sys/ioctl.h>
-#endif	/* !MSDOS */
+#endif
 
 #include "system.h"
 
@@ -40,64 +38,70 @@
 
 #ifdef	FD
 #include "term.h"
-extern int Xstat __P_((char *, struct stat *));
-extern int Xlstat __P_((char *, struct stat *));
-extern int Xaccess __P_((char *, int));
+extern int Xstat __P_((CONST char *, struct stat *));
+extern int Xlstat __P_((CONST char *, struct stat *));
+extern int Xaccess __P_((CONST char *, int));
+# ifndef	_NOPTY
+#include "termemu.h"
+extern VOID sendparent __P_((int, ...));
+# endif
 #else	/* !FD */
 extern int ttyio;
 extern FILE *ttyout;
 # if	MSDOS
-extern int Xstat __P_((char *, struct stat *));
+extern int Xstat __P_((CONST char *, struct stat *));
 # define	Xlstat		Xstat
 # else
-# define	Xstat		stat
-# define	Xlstat		lstat
+# define	Xstat(p, s)	((stat(p, s)) ? -1 : 0)
+# define	Xlstat(p, s)	((lstat(p, s)) ? -1 : 0)
 # endif
-#define	Xaccess(p, m)	(access(p, m) ? -1 : 0)
+#define	Xaccess(p, m)	((access(p, m)) ? -1 : 0)
 #endif	/* !FD */
 
 #ifndef	EPERM
 #define	EPERM		EACCES
 #endif
 
-extern VOID error __P_((char *));
+extern VOID error __P_((CONST char *));
 extern char *malloc2 __P_((ALLOC_T));
 extern char *realloc2 __P_((VOID_P, ALLOC_T));
-extern char *strdup2 __P_((char *));
-extern char *strndup2 __P_((char *, int));
-extern char *strchr2 __P_((char *, int));
-extern char *strncpy2 __P_((char *, char *, int));
-extern int setenv2 __P_((char *, char *, int));
+extern char *strdup2 __P_((CONST char *));
+extern char *strndup2 __P_((CONST char *, int));
+extern char *strchr2 __P_((CONST char *, int));
+extern char *strncpy2 __P_((char *, CONST char *, int));
+extern int setenv2 __P_((CONST char *, CONST char *, int));
 extern time_t time2 __P_((VOID_A));
 
 #ifndef	NOJOB
-static char *NEAR headstree __P_((syntaxtree *));
+static CONST char *NEAR headstree __P_((syntaxtree *));
 #endif
 #ifndef	NOALIAS
+static int NEAR searchalias __P_((CONST char *, int));
 static shaliastable *NEAR duplalias __P_((shaliastable *));
 static int cmpalias __P_((CONST VOID_P, CONST VOID_P));
 #endif
-static int NEAR _evalexpression __P_((char *, int, long *));
-static int NEAR evalexpression __P_((char *, int, long *, int));
+static int NEAR _evalexpression __P_((CONST char *, int, long *));
+static int NEAR evalexpression __P_((CONST char *, int, long *, int));
 #if	!MSDOS
-static VOID NEAR addmailpath __P_((char *, char *, time_t));
+static VOID NEAR addmailpath __P_((CONST char *, char *, time_t));
 #endif
-static int NEAR testsub1 __P_((int, char *, int *));
-static int NEAR testsub2 __P_((int, char *[], int *));
-static int NEAR testsub3 __P_((int, char *[], int *, int));
+static int NEAR testsub1 __P_((int, CONST char *, int *));
+static int NEAR testsub2 __P_((int, char *CONST *, int *));
+static int NEAR testsub3 __P_((int, char *CONST *, int *, int));
 
 #define	ALIASDELIMIT	"();&|<>"
 #define	BUFUNIT		32
-#define	getconstvar(s)	(getshellvar(s, sizeof(s) - 1))
+#define	getconstvar(s)	(getshellvar(s, strsize(s)))
 #define	ER_COMNOFOUND	1
 #define	ER_NOTFOUND	2
 #define	ER_NOTIDENT	4
 #define	ER_BADOPTIONS	13
-#define	ER_MISSARG	15
-#define	ER_NOTALIAS	19
-#define	ER_NOSUCHJOB	20
-#define	ER_NUMOUTRANGE	21
-#define	ER_UNKNOWNSIG	22
+#define	ER_NUMOUTRANGE	18
+#define	ER_NOTALIAS	20
+#define	ER_MISSARG	21
+#define	ER_NOSUCHJOB	22
+#define	ER_TERMINATED	23
+#define	ER_UNKNOWNSIG	26
 
 #if	!MSDOS
 typedef struct _mailpath_t {
@@ -108,21 +112,23 @@ typedef struct _mailpath_t {
 #endif
 
 #ifndef	NOJOB
-int gettermio __P_((p_id_t));
+int gettermio __P_((p_id_t, int));
 VOID dispjob __P_((int, FILE *));
 int searchjob __P_((p_id_t, int *));
-int getjob __P_((char *));
+int getjob __P_((CONST char *));
 int stackjob __P_((p_id_t, int, syntaxtree *));
 int stoppedjob __P_((p_id_t));
 VOID killjob __P_((VOID_A));
-VOID checkjob __P_((int));
+VOID checkjob __P_((FILE *));
 #endif	/* !NOJOB */
-char *evalposixsubst __P_((char *, int *));
+char *evalposixsubst __P_((CONST char *, int *));
 #if	!MSDOS
-VOID replacemailpath __P_((char *, int));
+VOID replacemailpath __P_((CONST char *, int));
 VOID checkmail __P_((int));
 #endif
 #ifndef	NOALIAS
+int addalias __P_((char *, char *));
+int deletealias __P_((CONST char *));
 VOID freealias __P_((shaliastable *));
 int checkalias __P_((syntaxtree *, char *, int, int));
 #endif
@@ -163,15 +169,16 @@ static int maxmailpath = 0;
 
 
 #ifndef	NOJOB
-int gettermio(pgrp)
+int gettermio(pgrp, job)
 p_id_t pgrp;
+int job;
 {
 	int ret;
 	sigmask_t mask, omask;
 
 	if (ttypgrp < (p_id_t)0 || pgrp < (p_id_t)0 || pgrp == ttypgrp)
 		return(0);
-	else if (!jobok) {
+	else if (!job) {
 		ttypgrp = pgrp;
 		return(0);
 	}
@@ -197,10 +204,11 @@ p_id_t pgrp;
 #endif	/* JOBVERBOSE */
 	if ((ret = settcpgrp(ttyio, pgrp)) >= 0) ttypgrp = pgrp;
 	Xsigsetmask(omask);
+
 	return(ret);
 }
 
-static char *NEAR headstree(trp)
+static CONST char *NEAR headstree(trp)
 syntaxtree *trp;
 {
 	int id;
@@ -216,6 +224,7 @@ syntaxtree *trp;
 	if (!trp || !(trp -> comm)) return(NULL);
 	id = (trp -> comm) -> id;
 	if (id == SM_CHILD || id == SM_STATEMENT) return(NULL);
+
 	return(statementlist[id - 1].ident);
 }
 
@@ -261,13 +270,14 @@ int *np;
 			return(i);
 		}
 	}
+
 	return(-1);
 }
 
 int getjob(s)
-char *s;
+CONST char *s;
 {
-	char *cp;
+	CONST char *cp;
 	int i, j;
 
 	if (!jobok) return(-1);
@@ -291,7 +301,7 @@ char *s;
 	if (i < 0 || i >= maxjobs || !joblist || !(joblist[i].pids))
 		return(-1);
 	j = joblist[i].npipe;
-	if (joblist[i].stats[j] < 0 || joblist[i].stats[j] >= 128) return(-1);
+	if (joblist[i].stats[j] < 0 || joblist[i].stats[j] >= 128) return(-2);
 
 	return(i);
 }
@@ -361,11 +371,7 @@ syntaxtree *trp;
 	joblist[i].pids[j] = pid;
 	joblist[i].stats[j] = sig;
 	if (!j && !(joblist[i].trp) && trp) {
-#ifdef	MINIMUMSHELL
-		joblist[i].trp = duplstree(trp, NULL);
-#else
 		joblist[i].trp = duplstree(trp, NULL, 0L);
-#endif
 		if (!isoppipe(joblist[i].trp) && joblist[i].trp -> next) {
 			freestree(joblist[i].trp -> next);
 			joblist[i].trp -> next = NULL;
@@ -378,6 +384,7 @@ syntaxtree *trp;
 		fprintf2(ttyout, "%id ", joblist[i].pids[j]);
 	fputnl(ttyout);
 #endif	/* JOBVERBOSE */
+
 	return(i);
 }
 
@@ -394,16 +401,17 @@ p_id_t pid;
 			else if (!sig) return(0);
 		}
 	}
+
 	return(-1);
 }
 
-VOID checkjob(verbose)
-int verbose;
+VOID checkjob(fp)
+FILE *fp;
 {
 	int i, j;
 
 	while (waitjob(-1, NULL, WNOHANG | WUNTRACED) > 0);
-	if (verbose) for (i = 0; i < maxjobs; i++) {
+	if (fp) for (i = 0; i < maxjobs; i++) {
 		if (!(joblist[i].pids)) continue;
 		j = joblist[i].npipe;
 		if (joblist[i].stats[j] >= 0 && joblist[i].stats[j] < 128)
@@ -412,7 +420,7 @@ int verbose;
 		if (joblist[i].trp
 		&& (isopbg(joblist[i].trp) || isopnown(joblist[i].trp)))
 			joblist[i].trp -> type = OP_NONE;
-		if (jobok && interactive && !nottyout) dispjob(i, stderr);
+		if (jobok && interactive && !nottyout) dispjob(i, fp);
 		free(joblist[i].pids);
 		free(joblist[i].stats);
 		if (joblist[i].trp) {
@@ -442,6 +450,76 @@ VOID killjob(VOID_A)
 #endif	/* !NOJOB */
 
 #ifndef	NOALIAS
+static int NEAR searchalias(ident, len)
+CONST char *ident;
+int len;
+{
+	int i;
+
+	if (len < 0) len = strlen(ident);
+	for (i = 0; shellalias[i].ident; i++)
+		if (!strncommcmp(ident, shellalias[i].ident, len)
+		&& !(shellalias[i].ident[len]))
+			break;
+
+	return(i);
+}
+
+int addalias(ident, comm)
+char *ident, *comm;
+{
+	int i;
+
+	i = searchalias(ident, -1);
+	if (shellalias[i].ident) {
+		free(shellalias[i].ident);
+		free(shellalias[i].comm);
+	}
+	else {
+		shellalias = (shaliastable *)realloc2(shellalias,
+			(i + 2) * sizeof(shaliastable));
+		shellalias[i + 1].ident = NULL;
+	}
+
+	shellalias[i].ident = ident;
+	shellalias[i].comm = comm;
+# if	defined (FD) && !defined (_NOPTY)
+	sendparent(TE_ADDALIAS, ident, comm);
+# endif
+
+	return(0);
+}
+
+int deletealias(ident)
+CONST char *ident;
+{
+	reg_t *re;
+	int i, n, max;
+
+	n = 0;
+	re = regexp_init(ident, -1);
+	for (max = 0; shellalias[max].ident; max++);
+	for (i = 0; i < max; i++) {
+		if (re) {
+			if (!regexp_exec(re, shellalias[i].ident, 0)) continue;
+		}
+		else if (strcommcmp(ident, shellalias[i].ident)) continue;
+		free(shellalias[i].ident);
+		free(shellalias[i].comm);
+		memmove((char *)&(shellalias[i]), (char *)&(shellalias[i + 1]),
+			(max-- - i) * sizeof(shaliastable));
+		i--;
+		n++;
+	}
+	if (re) regexp_free(re);
+	if (!n) return(-1);
+# if	defined (FD) && !defined (_NOPTY)
+	sendparent(TE_DELETEALIAS, ident);
+# endif
+
+	return(0);
+}
+
 static shaliastable *NEAR duplalias(alias)
 shaliastable *alias;
 {
@@ -456,6 +534,7 @@ shaliastable *alias;
 		dupl[i].comm = strdup2(alias[i].comm);
 	}
 	dupl[i].ident = NULL;
+
 	return(dupl);
 }
 
@@ -481,6 +560,7 @@ CONST VOID_P vp2;
 
 	ap1 = (shaliastable *)vp1;
 	ap2 = (shaliastable *)vp2;
+
 	return(strpathcmp2(ap1 -> ident, ap2 -> ident));
 }
 
@@ -494,24 +574,24 @@ int len, delim;
 	if (!trp || (trp -> flags & ST_NODE) || hascomm(trp)
 	|| !interactive
 	|| ((i = getstatid(parentstree(trp))) >= 0
-	&& !(statementlist[i].type & STT_NEEDLIST))) return(-1);
+	&& !(statementlist[i].type & STT_NEEDLIST)))
+		return(-1);
 
 	if ((!strchr(IFS_SET, delim) && !strchr(ALIASDELIMIT, delim)))
 		return(-1);
-	for (i = 0; shellalias[i].ident; i++)
-		if (!(shellalias[i].flags & AL_USED)
-		&& !strncommcmp(ident, shellalias[i].ident, len)
-		&& !(shellalias[i].ident[len])) break;
-	if (!(shellalias[i].ident)) return(-1);
+	i = searchalias(ident, len);
+	if (!(shellalias[i].ident) || (shellalias[i].flags & AL_USED))
+		return(-1);
 
 	*ident = '\0';
 	shellalias[i].flags |= AL_USED;
+
 	return(i);
 }
 #endif	/* NOALIAS */
 
 static int NEAR _evalexpression(s, ptr, resultp)
-char *s;
+CONST char *s;
 int ptr;
 long *resultp;
 {
@@ -576,11 +656,12 @@ long *resultp;
 	else return(-1);
 
 	while (s[ptr] && strchr(IFS_SET, s[ptr])) ptr++;
+
 	return(ptr);
 }
 
 static int NEAR evalexpression(s, ptr, resultp, lvl)
-char *s;
+CONST char *s;
 int ptr;
 long *resultp;
 int lvl;
@@ -724,10 +805,14 @@ int lvl;
 }
 
 char *evalposixsubst(s, ptrp)
-char *s;
+CONST char *s;
 int *ptrp;
 {
-	char *cp;
+#ifndef	BASHBUG
+	syntaxtree *trp, *stree;
+	redirectlist red;
+#endif
+	char *cp, *new;
 	long n;
 	int i, len;
 
@@ -736,18 +821,15 @@ int *ptrp;
 
 	s += i;
 	len = *ptrp - i * 2;
-	s = strndup2(s, len);
+	s = new = strndup2(s, len);
 	if (i <= 1) {
 #ifndef	BASHBUG
 	/* bash cannot include 'case' statement within $() */
-		syntaxtree *trp, *stree;
-		redirectlist red;
-
 		red.fd = red.type = red.new = 0;
-		red.filename = "";
+		red.filename = vnullstr;
 		i = len = 0;
 		stree = newstree(NULL);
-		trp = startvar(stree, &red, "", &i, &len, 0);
+		trp = startvar(stree, &red, nullstr, &i, &len, 0);
 		trp -> cont = CN_COMM;
 		trp = analyze(s, trp, -1);
 		if (trp && (trp -> cont & CN_SBST) != CN_CASE) trp = NULL;
@@ -760,7 +842,7 @@ int *ptrp;
 	}
 	else {
 		n = 0L;
-		if (!(cp = evalvararg(s, '\'', EA_BACKQ, 0))) *ptrp = -1;
+		if (!(cp = evalvararg(new, '\'', EA_BACKQ, 0))) *ptrp = -1;
 		else {
 			for (i = 0; cp[i] && strchr(IFS_SET, cp[i]); i++);
 			if (cp[i]) {
@@ -773,14 +855,15 @@ int *ptrp;
 		if (*ptrp < 0) cp = NULL;
 		else if (asprintf2(&cp, "%ld", n) < 0) error("malloc()");
 	}
-	free(s);
+	free(new);
 
 	return(cp);
 }
 
 #if	!MSDOS
 static VOID NEAR addmailpath(path, msg, mtime)
-char *path, *msg;
+CONST char *path;
+char *msg;
 time_t mtime;
 {
 	int i;
@@ -801,12 +884,13 @@ time_t mtime;
 }
 
 VOID replacemailpath(s, multi)
-char *s;
+CONST char *s;
 int multi;
 {
 	mailpath_t *mailpath;
 	time_t mtime;
-	char *cp, *msg, path[MAXPATHLEN];
+	CONST char *cp;
+	char *msg, path[MAXPATHLEN];
 	int i, max, len;
 
 	mailpath = mailpathlist;
@@ -824,7 +908,7 @@ int multi;
 		if ((cp = strchr(s, ':'))) len = (cp++) - s;
 		else {
 			len = strlen(s);
-			cp = s + len;
+			cp = &(s[len]);
 		}
 		if ((msg = strchr(s, '%'))) {
 			if (msg > s + len) msg = NULL;
@@ -861,7 +945,7 @@ int reset;
 			free(mailpathlist[i].path);
 			if (mailpathlist[i].msg) free(mailpathlist[i].msg);
 		}
-		free(mailpathlist);
+		if (mailpathlist) free(mailpathlist);
 		mailpathlist = NULL;
 		maxmailpath = 0;
 		return;
@@ -887,7 +971,7 @@ syntaxtree *trp;
 	int i, j;
 
 	if (mypid != orgpgrp) return(RET_SUCCESS);
-	checkjob(0);
+	checkjob(NULL);
 	for (i = 0; i < maxjobs; i++) {
 		if (!(joblist[i].pids)) continue;
 		j = joblist[i].npipe;
@@ -907,6 +991,7 @@ syntaxtree *trp;
 			joblist[i].pids = NULL;
 		}
 	}
+
 	return(RET_SUCCESS);
 }
 
@@ -919,9 +1004,10 @@ syntaxtree *trp;
 	int i, j, n, ret;
 
 	s = ((trp -> comm) -> argc > 1) ? (trp -> comm) -> argv[1] : NULL;
-	checkjob(0);
+	checkjob(NULL);
 	if ((i = getjob(s)) < 0) {
-		execerror((trp -> comm) -> argv[1], ER_NOSUCHJOB, 0);
+		execerror((trp -> comm) -> argv[1],
+			(i < -1) ? ER_TERMINATED : ER_NOSUCHJOB, 2);
 		return(RET_FAIL);
 	}
 	n = joblist[i].npipe;
@@ -936,7 +1022,7 @@ syntaxtree *trp;
 # ifdef	USESGTTY
 	if (joblist[i].ttyflag) ioctl(ttyio, TIOCLSET, joblist[i].ttyflag);
 # endif
-	gettermio(joblist[i].pids[0]);
+	gettermio(joblist[i].pids[0], jobok);
 	if ((j = joblist[i].stats[n]) > 0 && j < 128) {
 		Xkillpg(joblist[i].pids[0], SIGCONT);
 		for (j = 0; j <= n; j++) joblist[i].stats[j] = 0;
@@ -944,12 +1030,13 @@ syntaxtree *trp;
 	if (isopbg(joblist[i].trp) || isopnown(joblist[i].trp))
 		joblist[i].trp -> type = OP_NONE;
 	ret = waitchild(joblist[i].pids[n], NULL);
-	gettermio(orgpgrp);
+	gettermio(orgpgrp, jobok);
 	if (tioctl(ttyio, REQSETP, &tty) < 0) {
 		doperror((trp -> comm) -> argv[0], "fatal error");
 		prepareexit(-1);
 		Xexit(RET_FATALERR);
 	}
+
 	return(ret);
 }
 
@@ -961,9 +1048,10 @@ syntaxtree *trp;
 	int i, j, n;
 
 	s = ((trp -> comm) -> argc > 1) ? (trp -> comm) -> argv[1] : NULL;
-	checkjob(0);
+	checkjob(NULL);
 	if ((i = getjob(s)) < 0) {
-		execerror((trp -> comm) -> argv[1], ER_NOSUCHJOB, 0);
+		execerror((trp -> comm) -> argv[1],
+			(i < -1) ? ER_TERMINATED : ER_NOSUCHJOB, 2);
 		return(RET_FAIL);
 	}
 	n = joblist[i].npipe;
@@ -976,6 +1064,7 @@ syntaxtree *trp;
 	if (interactive && !nottyout) dispjob(i, stderr);
 	prevjob = lastjob;
 	lastjob = i;
+
 	return(RET_SUCCESS);
 }
 #endif	/* !NOJOB */
@@ -1009,16 +1098,15 @@ syntaxtree *trp;
 			set = 1;
 			break;
 		}
-		for (i = 0; shellalias[i].ident; i++)
-			if (!strncommcmp(shellalias[i].ident, argv[n], len)
-			&& !(shellalias[i].ident[len]))
-				break;
 
-		if (!set) {
+		if (set) VOID_C addalias(strndup2(argv[n], len),
+			strdup2(&(argv[n][len + 1])));
+		else {
+			i = searchalias(argv[n], len);
 			if (!(shellalias[i].ident)) {
 				if (interactive) {
 					fputs("alias: ", stderr);
-					execerror(argv[n], ER_NOTFOUND, 0);
+					execerror(argv[n], ER_NOTFOUND, 2);
 				}
 				ret = RET_FAIL;
 				ERRBREAK;
@@ -1027,59 +1115,30 @@ syntaxtree *trp;
 				shellalias[i].ident, shellalias[i].comm);
 			fputnl(stdout);
 		}
-		else if (shellalias[i].ident) {
-			free(shellalias[i].comm);
-			shellalias[i].comm = strdup2(&(argv[n][++len]));
-		}
-		else {
-			shellalias = (shaliastable *)realloc2(shellalias,
-				(i + 2) * sizeof(shaliastable));
-			shellalias[i].ident = strndup2(argv[n], len);
-			shellalias[i].comm = strdup2(&(argv[n][++len]));
-			shellalias[++i].ident = NULL;
-		}
 	}
+
 	return(ret);
 }
 
 int posixunalias(trp)
 syntaxtree *trp;
 {
-	reg_t *re;
 	char **argv;
-	int i, n, c, ret;
+	int n, ret;
 
 	argv = (trp -> comm) -> argv;
 	ret = RET_SUCCESS;
 	for (n = 1; n < (trp -> comm) -> argc; n++) {
-		c = 0;
-		re = regexp_init(argv[n], -1);
-		for (i = 0; shellalias[i].ident; i++) {
-			if (re) {
-				if (!regexp_exec(re, shellalias[i].ident, 0))
-					continue;
-			}
-			else if (strcommcmp(shellalias[i].ident, argv[n]))
-				continue;
-			c++;
-			free(shellalias[i].ident);
-			free(shellalias[i].comm);
-			for (; shellalias[i + 1].ident; i++) {
-				shellalias[i].ident = shellalias[i + 1].ident;
-				shellalias[i].comm = shellalias[i + 1].comm;
-			}
-			shellalias[i].ident = NULL;
-		}
-		if (re) regexp_free(re);
-		if (!c) {
+		if (deletealias(argv[n]) < 0) {
 			if (interactive) {
 				fputs("alias: ", stderr);
-				execerror(argv[n], ER_NOTALIAS, 0);
+				execerror(argv[n], ER_NOTALIAS, 2);
 			}
 			ret = RET_FAIL;
 			ERRBREAK;
 		}
 	}
+
 	return(ret);
 }
 #endif	/* NOALIAS */
@@ -1123,27 +1182,28 @@ syntaxtree *trp;
 				if (!strcmp(&(s[1]), signallist[i].ident))
 					break;
 			if (signallist[i].sig < 0) {
-				execerror(&(s[1]), ER_UNKNOWNSIG, 0);
+				execerror(&(s[1]), ER_UNKNOWNSIG, 1);
 				return(RET_FAIL);
 			}
 			sig = signallist[i].sig;
 		}
 		if (sig < 0 || sig >= NSIG) {
-			execerror(s, ER_NUMOUTRANGE, 0);
+			execerror(s, ER_NUMOUTRANGE, 1);
 			return(RET_FAIL);
 		}
 		i = 2;
 	}
 
 #ifndef	NOJOB
-	checkjob(0);
+	checkjob(NULL);
 #endif
 	for (; i < (trp -> comm) -> argc; i++) {
 		s = (trp -> comm) -> argv[i];
 #ifndef	NOJOB
 		if (*s == '%') {
 			if ((n = getjob(s)) < 0) {
-				execerror(s, ER_NOSUCHJOB, 0);
+				execerror(s, (n < -1)
+					? ER_TERMINATED : ER_NOSUCHJOB, 2);
 				return(RET_FAIL);
 			}
 			n = Xkillpg(joblist[n].pids[0], sig);
@@ -1169,12 +1229,13 @@ syntaxtree *trp;
 			return(RET_FAIL);
 		}
 	}
+
 	return(RET_SUCCESS);
 }
 
 static int NEAR testsub1(c, s, ptrp)
 int c;
-char *s;
+CONST char *s;
 int *ptrp;
 {
 	struct stat st;
@@ -1237,7 +1298,7 @@ int *ptrp;
 #if	defined (BASHSTYLE) || defined (STRICTPOSIX)
 	/* "test" on bash & POSIX has -e & -L option */
 		case 'e':
-			if (s) ret = (*s && Xstat(s, &st) >= 0)
+			if (s) ret = (*s && Xlstat(s, &st) >= 0)
 				? RET_SUCCESS : RET_FAIL;
 			break;
 		case 'L':
@@ -1284,15 +1345,16 @@ int *ptrp;
 			break;
 	}
 	if (ret >= 0) *ptrp += 2;
+
 	return(ret);
 }
 
 static int NEAR testsub2(argc, argv, ptrp)
 int argc;
-char *argv[];
+char *CONST *argv;
 int *ptrp;
 {
-	char *s, *a1, *a2;
+	CONST char *s, *a1, *a2;
 	int ret, v1, v2;
 
 	if (*ptrp >= argc) return(-1);
@@ -1301,7 +1363,8 @@ int *ptrp;
 		if ((ret = testsub3(argc, argv, ptrp, 0)) < 0)
 			return(ret);
 		if (*ptrp >= argc
-		|| argv[*ptrp][0] != ')' || argv[*ptrp][1]) return(-1);
+		|| argv[*ptrp][0] != ')' || argv[*ptrp][1])
+			return(-1);
 		(*ptrp)++;
 		return(ret);
 	}
@@ -1362,12 +1425,13 @@ int *ptrp;
 	}
 	ret = (argv[*ptrp][0]) ? RET_SUCCESS : RET_FAIL;
 	(*ptrp)++;
+
 	return(ret);
 }
 
 static int NEAR testsub3(argc, argv, ptrp, lvl)
 int argc;
-char *argv[];
+char *CONST *argv;
 int *ptrp, lvl;
 {
 	int ret1, ret2;
@@ -1393,13 +1457,14 @@ int *ptrp, lvl;
 		? (ret1 != RET_FAIL && ret2 != RET_FAIL)
 		: (ret1 != RET_FAIL || ret2 != RET_FAIL))
 			? RET_SUCCESS : RET_FAIL;
+
 	return(ret1);
 }
 
 int posixtest(trp)
 syntaxtree *trp;
 {
-	char **argv;
+	char *CONST *argv;
 	int ret, ptr, argc;
 
 	argc = (trp -> comm) -> argc;
@@ -1432,6 +1497,7 @@ syntaxtree *trp;
 		fputnl(stderr);
 		return(RET_NOTICE);
 	}
+
 	return(ret);
 }
 
@@ -1484,7 +1550,7 @@ syntaxtree *trp;
 # endif
 				if ((type & CM_NOTFOUND)
 				|| Xaccess(cp, X_OK) < 0) {
-					execerror(cp, ER_COMNOFOUND, 0);
+					execerror(cp, ER_COMNOFOUND, 1);
 					cp = NULL;
 				}
 			}
@@ -1521,18 +1587,19 @@ syntaxtree *trp;
 		ret = exec_simplecom(trp, type, id, 0);
 #endif
 	else {
-		path = strdup2(getconstvar("PATH"));
-		setenv2("PATH", DEFPATH, 1);
+		path = strdup2(getconstvar(ENVPATH));
+		setenv2(ENVPATH, DEFPATH, 1);
 #if	MSDOS
 		ret = exec_simplecom(trp, type, id);
 #else
 		ret = exec_simplecom(trp, type, id, 0);
 #endif
-		setenv2("PATH", path, 1);
+		setenv2(ENVPATH, path, 1);
 		if (path) free(path);
 	}
 	(trp -> comm) -> argc = argc;
 	(trp -> comm) -> argv = argv;
+
 	return(ret);
 }
 
@@ -1547,7 +1614,7 @@ syntaxtree *trp;
 	argv = (trp -> comm) -> argv;
 
 	if (argc <= 2) {
-		execerror(NULL, ER_MISSARG, 0);
+		execerror(NULL, ER_MISSARG, 1);
 		return(RET_FAIL);
 	}
 	if (identcheck(argv[2], '\0') <= 0) {
@@ -1627,8 +1694,8 @@ syntaxtree *trp;
 		}
 	}
 
-	if (posixoptarg) setenv2("OPTARG", posixoptarg, 0);
-	else if (ret == RET_SUCCESS) unset("OPTARG", sizeof("OPTARG") - 1);
+	if (posixoptarg) setenv2(ENVOPTARG, posixoptarg, 0);
+	else if (ret == RET_SUCCESS) unset(ENVOPTARG, strsize(ENVOPTARG));
 
 	buf[0] = n;
 	buf[1] = '\0';
@@ -1636,7 +1703,7 @@ syntaxtree *trp;
 
 	n = posixoptind;
 	snprintf2(buf, sizeof(buf), "%d", n);
-	setenv2("OPTIND", buf, 0);
+	setenv2(ENVOPTIND, buf, 0);
 	posixoptind = n;
 
 	return(ret);
